@@ -1,6 +1,6 @@
 extern crate classreader;
 
-use classreader as c;
+//use classreader as c;
 
 fn main() {
     //let cls = c::ClassReader::new_from_path("/home/werner/test/pkg/Test.class").unwrap();
